@@ -39,9 +39,16 @@ fill_graphics_with_brush (GpGraphics *graphics, GpBrush *brush, BOOL stroke)
 	/* We do brush setup just before filling. */
 	gdip_brush_setup (graphics, brush);
 
-	if (stroke)
+	/* don't stroke if scaled (since the pen thickness will be scaled too!) */
+	if (stroke && !gdip_is_scaled (graphics)) {
+		/* stroke only using 1 pixel width - see #413461 */
+		double width = cairo_get_line_width (graphics->ct);
+		cairo_set_line_width (graphics->ct, 1.0);
 		cairo_stroke_preserve (graphics->ct);
+		cairo_set_line_width (graphics->ct, width);
+	}
 
+	cairo_close_path (graphics->ct);
 	cairo_fill (graphics->ct);
 
 	/* Set the matrix back to graphics->copy_of_ctm for other functions.
@@ -95,7 +102,7 @@ make_ellipse (GpGraphics *graphics, float x, float y, float width, float height,
 	cy = y + ry;
 
 	/* if required deal, once and for all, with antialiasing */
-	if (antialiasing) {
+	if (antialiasing && !gdip_is_scaled (graphics)) {
 		cx += graphics->aa_offset_x;
 		cy += graphics->aa_offset_y;
 	}
@@ -196,8 +203,7 @@ make_arcs (GpGraphics *graphics, float x, float y, float width, float height, fl
 {
 	int i;
 	float drawn = 0;
-	float endAngle = startAngle + sweepAngle;	
-	int increment = (endAngle > 0) ? 90 : -90;
+	float endAngle;
 	BOOL enough = FALSE;
 
 	/* if required deal, once and for all, with unit conversions */
@@ -214,6 +220,14 @@ make_arcs (GpGraphics *graphics, float x, float y, float width, float height, fl
 		return;
 	}
 
+	endAngle = startAngle + sweepAngle;
+	/* if we end before the start then reverse positions (to keep increment positive) */
+	if (endAngle < startAngle) {
+		double temp = endAngle;
+		endAngle = startAngle;
+		startAngle = temp;
+	}
+
 	/* i is the number of sub-arcs drawn, each sub-arc can be at most 90 degrees.*/
 	/* there can be no more then 4 subarcs, ie. 90 + 90 + 90 + (something less than 90) */
 	for (i = 0; i < 4; i++) {
@@ -222,21 +236,20 @@ make_arcs (GpGraphics *graphics, float x, float y, float width, float height, fl
 
 		if (enough)
 			return;
-		
-		if (fabs (current + increment) < fabs (endAngle))
-			additional = increment; /* add the default increment */
-		else {
-			additional = endAngle - current; /* otherwise, add the remainder */
+
+		additional = endAngle - current; /* otherwise, add the remainder */
+		if (additional > 90) {
+			additional = 90.0;
+		} else {
+			/* a near zero value will introduce bad artefact in the drawing (#78999) */
+			if (gdip_near_zero (additional))
+				return;
+
 			enough = TRUE;
 		}
 
-		/* a near zero value will introduce bad artefact in the drawing (#78999) */
-		if (gdip_near_zero (additional))
-			return;
-
-		make_arc (graphics,
-			  (i == 0) ? TRUE : FALSE,  /* only move to the starting pt in the 1st iteration */
-			  x, y, width, height,      /* bounding rectangle */
+		make_arc (graphics, (i == 0),	/* only move to the starting pt in the 1st iteration */
+			  x, y, width, height,	/* bounding rectangle */
 			  current, current + additional, antialiasing);
 		drawn += additional;		
 	}
@@ -617,7 +630,7 @@ make_pie (GpGraphics *graphics, float x, float y, float width, float height,
 	cos_alpha = cos (alpha);
 
 	/* if required deal, once and for all, with antialiasing */
-	if (antialiasing) {
+	if (antialiasing && !gdip_is_scaled (graphics)) {
 		cx += graphics->aa_offset_x;
 		cy += graphics->aa_offset_y;
 	}
